%
% (c) The GRASP/AQUA Project, Glasgow University, 1995-1999
%
\section[PosixUtil]{(Glasgow) Haskell POSIX utilities}

\begin{code}
{-# OPTIONS -#include "HsPosix.h" #-}

module PosixUtil where

import System.Posix.Types
import GlaExts
import GHC.IOBase  -- IOError representation
\end{code}

\begin{code}
type Limit = Int

instance CCallable ByteCount
instance CCallable ClockTick
instance CCallable EpochTime
instance CCallable LinkCount
instance CCallable DeviceID
instance CCallable FileID
instance CCallable FileMode
instance CCallable ProcessID
instance CCallable UserID
instance CCallable GroupID
instance CCallable FileOffset
instance CCallable Fd

instance CReturnable ByteCount
instance CReturnable ClockTick
instance CReturnable EpochTime
instance CReturnable LinkCount
instance CReturnable DeviceID
instance CReturnable FileID
instance CReturnable FileMode
instance CReturnable ProcessID
instance CReturnable UserID
instance CReturnable GroupID
instance CReturnable FileOffset
instance CReturnable Fd
\end{code}

Now some local functions that shouldn't go outside this library.

Fail with a SystemError.  Normally, we do not try to re-interpret
POSIX error numbers, so most routines in this file will only fail
with SystemError.  The only exceptions are (1) those routines where
failure of some kind may be considered ``normal''...e.g. getpwnam()
for a non-existent user, or (2) those routines which do not set
errno.

\begin{code}
syserr :: String -> IO a
syserr str = ioException (IOError Nothing     -- ToDo: better
			      SystemError
			      str
			      ""
			      Nothing)

-- common templates for system calls

nonzero_error :: IO Int -> String -> IO ()
nonzero_error io err = do
    rc <- io
    if rc == 0
       then return ()
       else syserr err

minusone_error :: IO Int -> String -> IO ()
minusone_error io err = do
    rc <- io
    if rc /= -1
       then return ()
       else syserr err

\end{code}
